using System;

using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

using Syncfusion.Windows.Forms.Diagram;

namespace Symcon.Lightridge.LRRUI.Controls
{
	[
	ToolboxItem(true),
	Description("Property Editor control for nodes in a canvas.")
	]
	public class PropertyEditor : Panel
	{
		private System.ComponentModel.IContainer components;
		private Canvas					canvas = null;
		private PropertyGrid			propGrid;
		public PropertyEditor()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// Create embedded property grid and configure it.
			this.propGrid = new PropertyGrid();
			this.Controls.Add(this.propGrid);
			this.propGrid.Dock = DockStyle.Fill;
			this.propGrid.PropertyValueChanged += new PropertyValueChangedEventHandler(this.PropertyGrid_ValueChanged);
		}


		/// <summary>
		/// Selects the list of nodes in the property grid.
		/// </summary>
		/// <param name="nodes">Nodes to display in the property editor.</param>
		public void SetSelectedObjects(NodeCollection nodes)
		{
			if (nodes.Count > 0)
			{
				Type nodeType = nodes[0].GetType();
				this.propGrid.SelectedObject = nodeType.GetProperty("PT").GetValue(nodes[0], null);
			}
			else
			{
				this.propGrid.SelectedObject = null;
			}
		}

		/// <summary>
		/// Select the node in the property grid.
		/// </summary>
		/// <param name="node">Node to display in the property editor.</param>
		public void SetSelectedObject(INode node)
		{
			if(node != null)
			{
				Type nodeType = node.GetType();
				if(nodeType == typeof(Model))
					this.propGrid.SelectedObject = this.canvas.Model;
				else
				this.propGrid.SelectedObject = nodeType.GetProperty("PT").GetValue(node, null);
			}
		}

		/// <summary>
		/// Reference to the PropertyGrid object contained by this property editor.
		/// </summary>
		public System.Windows.Forms.PropertyGrid PropertyGrid
		{
			get
			{
				return this.propGrid;
			}
		}

		/// <summary>
		/// canvas the property editor is attached to.
		/// </summary>
		/// <remarks>
		/// <para>
		/// This property contains a reference to the canvas that this property
		/// editor is attached to. The property editor receives events from the
		/// canvas when the current selection changes and it updates the currently
		/// displayed object in the property editor.
		/// </para>
		/// </remarks>
		[
		Browsable(true),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)
		]
		public Canvas Canvas
		{
			get
			{
				return this.canvas;
			}
			set
			{
				if (this.canvas != value)
				{
					if (this.canvas != null)
					{
						this.canvas.SelectionChanged -= new NodeCollectionEventHandler(canvas_SelectionChanged);
						this.canvas.PropertyChanged -= new PropertyEventHandler(canvas_PropertyChanged);

						this.canvas.Disposed+=new EventHandler(canvas_Disposed);
						this.canvas.NodeMoved -= new MoveEventHandler(canvas_NodeMoved);
					}

					this.canvas = value;

					if (this.canvas != null)
					{
						this.canvas.SelectionChanged += new NodeCollectionEventHandler(canvas_SelectionChanged);
						this.canvas.PropertyChanged += new PropertyEventHandler(canvas_PropertyChanged);
						this.canvas.NodeMoved += new MoveEventHandler(canvas_NodeMoved);

						if (!this.DesignMode)
						{
							this.SetSelectedObject(this.canvas.Model);
						}
					}
				}
			}
		}

		private void canvas_SelectionChanged(object sender, NodeCollectionEventArgs evtArgs)
		{
			if (!this.DesignMode)
			{
				if (evtArgs.Nodes == null || evtArgs.Nodes.Length == 0 || evtArgs.Nodes[0] == null)
				{
					this.propGrid.SelectedObject = this.canvas.View;
					return;
				}

				if(canvas.SelectionList.Count == 1 && canvas.SelectionList.First.GetType() == typeof(DynamicGridcontainer))
				{
					DynamicGridcontainer dg = canvas.SelectionList.First as DynamicGridcontainer;
					if(dg.SelectedCell != null)
					{
						this.propGrid.SelectedObject = dg.SelectedCell.PT;
					}
					else
						this.propGrid.SelectedObject = dg.PT;
				}
				else if(canvas.SelectionList.Count == 1 && canvas.SelectionList.First.GetType() == typeof(GridCell))
				{
					GridCell gc = canvas.SelectionList.First as GridCell;
					if(gc != null)
					{
						this.propGrid.SelectedObject = gc.PT;
					}
				}
                /*else if (canvas.SelectionList.Count == 1 && canvas.SelectionList.First.GetType() == typeof(Symcon.Lightridge.LRRUI.Controls.DashBoardControlProperties))
                {
                    Symbol smbl = canvas.SelectionList.Last as Symbol;
                    ControlNode ctrlNode = smbl.Nodes.Last as ControlNode;
                    Control ctrl = (Control)ctrlNode.GetType().GetProperty("Control").GetValue(ctrlNode, null);
                    this.propGrid.SelectedObject = ctrl.GetType().GetProperty("PT").GetValue(ctrl, null);
                }*/
				else
				{
					//this.propGrid.SelectedObjects = null;
					object[] selectedObjects = new object[canvas.SelectionList.Count];
					for(int i = 0; i < canvas.SelectionList.Count;i++)
					{
						if(canvas.SelectionList[i].GetType().BaseType == typeof(Symbol))
						{
							Type nodeType = canvas.SelectionList[i].GetType();
							selectedObjects[i] = nodeType.GetProperty("PT").GetValue(canvas.SelectionList[i], null);
							//	this.propGrid.SelectedObject = nodeType.GetProperty("PT").GetValue(evtArgs.Nodes[0], null);;
						}
						else 
						{
							Type nodeType = canvas.SelectionList[i].GetType();
							selectedObjects[i] = canvas.SelectionList[i];
						}
					}
					this.propGrid.SelectedObjects = selectedObjects;
				}
			}
		}

		private void canvas_PropertyChanged(object sender, PropertyEventArgs evtArgs)
		{
			if (!this.DesignMode)
			{
				//this.propGrid.Refresh();
			}
		}

		private void canvas_NodeMoved(object sender, MoveEventArgs evtArgs)
		{
			if (!this.DesignMode)
			{
				this.propGrid.Refresh();
			}
		}

		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(PropertyEditor));
		}

		private void PropertyGrid_ValueChanged(object sender, PropertyValueChangedEventArgs evtArgs)
		{
			if (this.canvas != null)
			{
				//this.canvas.View.Update();
			}
		}

		private void canvas_Disposed(object sender, EventArgs e)
		{
			//this.propGrid.SelectedObject = null;
		}
	}
}